<tr class="{{ extra_class }}">
    <th>{{ label }}</th>
    <td>
        <div class="bar" style="width:{{ width }}%" data-toggle="tooltip" data-placement="right" data-html="true"
             data-delay=500 title="Percentage: {{ percentage }}%">
            {{ label_in_bar }}
        </div>
        {{ label_after_bar }}
    </td>
</tr>