{% include '_row_header_ignore.html' %}
<div class="col-md-6">
    <p><em>This variable is a recoding of {{ values['correlation_var'] }} and should be ignored for analysis</em></p>
</div>

{% include '_row_footer.html' %}